@@ -18,7 +18,8 @@ import ctypes.util
 import os
 import sys
 import time
-import copy
+from copy import copy
+from  math import log10
 import atexit
 import warnings
 from threading import Thread
@@ -37,49 +38,16 @@ except ImportError:
 
 from . import dbr
 
-EPICS_STR_ENCODING = 'ASCII'
-PY_VERSION = sys.version_info[0]
-def get_strconvertors():
-    """create string wrappers to pass to C functions for both
-    Python2 and Python3.  Note that the EPICS CA library uses
-    char* to represent strings.  In Python3, char* maps to a
-    sequence of bytes which must be explicitly converted to a
-    Python string by specifying the encoding.  That is, ASCII
-    encoding is not implicitly assumed.
+PY_MAJOR, PY_MINOR = sys.version_info[:2]
 
-    That is, for Python3 one sends and receives sequences of
-    bytes to libca. This function returns the translators
-    (STR2BYTES, BYTES2STR), assuming the encoding defined in
-    EPICS_STR_ENCODING (which is 'ASCII' by default).
-    """
-    if PY_VERSION >= 3:
-        def s2b(st1):
-            'string to byte'
-            if isinstance(st1, bytes):
-                return st1
-            return bytes(st1, EPICS_STR_ENCODING)
-        def b2s(st1):
-            'byte to string'
-            if isinstance(st1, str):
-                return st1
-            return str(st1, EPICS_STR_ENCODING)
-        return s2b, b2s
-    return str, str
-
-STR2BYTES, BYTES2STR = get_strconvertors()
-
-def strjoin(sep, seq):
-    "join string sequence with a separator"
-    if PY_VERSION < 3:
-        return sep.join(seq)
-
-    if isinstance(sep, bytes):
-        sep = BYTES2STR(sep)
-    if len(seq) == 0:
-        seq = ''
-    elif isinstance(seq[0], bytes):
-        seq = [BYTES2STR(i) for i in seq]
-    return sep.join(seq)
+memcopy = copy
+if PY_MAJOR >= 3:
+    from .utils3 import STR2BYTES, BYTES2STR, NULLCHAR, NULLCHAR_2, strjoin
+else:
+    from .utils2 import STR2BYTES, BYTES2STR, NULLCHAR, NULLCHAR_2, strjoin
+    if PY_MINOR == 5:
+        def memcopy(a):
+            return a
 
 ## print to stdout
 def write(msg, newline=True, flush=True):
@@ -97,13 +65,12 @@ initial_context = None
 ## PREEMPTIVE_CALLBACK determines the CA context
 PREEMPTIVE_CALLBACK = True
 
-
 AUTO_CLEANUP = True
 
 ##
 # maximum element count for auto-monitoring of PVs in epics.pv
 # and for automatic conversion of numerical array data to numpy arrays
-AUTOMONITOR_MAXLENGTH = 16384
+AUTOMONITOR_MAXLENGTH = 65536 # 16384
 
 ## default timeout for connection
 #   This should be kept fairly short --
@@ -122,14 +89,19 @@ _cache  = {}
 ## Cache of pvs waiting for put to be done.
 _put_done =  {}
 
+# get a unique python value that cannot be a value held by an
+# actual PV to signal "Get is incomplete, awaiting callback"
+class Empty:
+    """used to create a unique python value that cannot be
+    held as an actual PV value"""
+    pass
+GET_PENDING = Empty()
+
 class ChannelAccessException(Exception):
     """Channel Access Exception: General Errors"""
-    def __init__(self, fcn, msg):
-        Exception.__init__(self)
-        self.fcn = fcn
-        self.msg = msg
-    def __str__(self):
-        return " %s returned '%s'" % (self.fcn, self.msg)
+    def __init__(self, *args):
+        Exception.__init__(self, *args)
+        sys.excepthook(*sys.exc_info())
 
 class CASeverityException(Exception):
     """Channel Access Severity Check Exception:
@@ -145,36 +117,49 @@ def find_libca():
     """
     find location of ca dynamic library
     """
-    search_path = [os.path.split( os.path.abspath(__file__))[0]]
-    search_path.extend(sys.path)
+    # Test 1: if PYEPICS_LIBCA env var is set, use it.
+    dllpath = os.environ.get('PYEPICS_LIBCA', None)
+    if (dllpath is not None and os.path.exists(dllpath) and
+        os.path.isfile(dllpath)):
+        return dllpath
+
+    # Test 2: look through Python path and PATH env var for dll
     path_sep = ':'
+    dylib   = 'lib'
     # For windows, we assume the DLLs are installed with the library
     if os.name == 'nt':
         path_sep = ';'
-        search_path.append(os.path.join(sys.prefix, 'DLLs'))
+        dylib = 'DLLs'
 
-    search_path.extend(os.environ['PATH'].split(path_sep))
+    _path = [os.path.split(os.path.abspath(__file__))[0],
+             os.path.split(os.path.dirname(os.__file__))[0],
+             os.path.join(sys.prefix, dylib)]
+
+    search_path = []
+    for adir in (_path + sys.path +
+                 os.environ.get('PATH','').split(path_sep) +
+                 os.environ.get('LD_LIBRARY_PATH','').split(path_sep) +
+                 os.environ.get('DYLD_LIBRARY_PATH','').split(path_sep)):
+        if adir not in search_path and os.path.isdir(adir):
+            search_path.append(adir)
 
     os.environ['PATH'] = path_sep.join(search_path)
 
-    # first, try the ctypes utility, which *should* work
-    # with LD_LIBRARY_PATH or ldconfig
+    # with PATH set above, the ctypes utility, find_library *should*
+    # find the dll....
     dllpath  = ctypes.util.find_library('ca')
     if dllpath is not None:
         return dllpath
 
-    ## OK, simplest version didn't work, look explicity through path
-    known_hosts = {'Linux':   ('linux-x86', 'linux-x86_64') ,
-                   'Darwin':  ('darwin-ppc', 'darwin-x86'),
-                   'SunOS':   ('solaris-sparc', 'solaris-sparc-gnu') }
-
-
+    # Test 3: on unixes, look expliticly with EPICS_BASE env var and
+    # known architectures for ca.so q
     if os.name == 'posix':
-        libname = 'libca.so'
-        ldpath = os.environ.get('LD_LIBRARY_PATH', '').split(':')
+        known_hosts = {'Linux':   ('linux-x86', 'linux-x86_64') ,
+                       'Darwin':  ('darwin-ppc', 'darwin-x86'),
+                       'SunOS':   ('solaris-sparc', 'solaris-sparc-gnu') }
 
+        libname = 'libca.so'
         if sys.platform == 'darwin':
-            ldpath = os.environ.get('DYLD_LIBRARY_PATH', '').split(':')
             libname = 'libca.dylib'
 
         epics_base = os.environ.get('EPICS_BASE', '.')
@@ -183,13 +168,12 @@ def find_libca():
             epicspath = []
             for adir in known_hosts[host_arch]:
                 epicspath.append(os.path.join(epics_base, 'lib', adir))
-        for adir in search_path + ldpath + epicspath + sys.path:
+        for adir in search_path + epicspath:
             if os.path.exists(adir) and os.path.isdir(adir):
                 if libname in os.listdir(adir):
                     return os.path.join(adir, libname)
 
-    raise ChannelAccessException('find_libca',
-                                 'Cannot find Epics CA DLL')
+    raise ChannelAccessException('cannot find Epics CA DLL')
 
 def initialize_libca():
     """ load DLL (shared object library) to establish Channel Access
@@ -213,14 +197,12 @@ def initialize_libca():
     try:
         libca = load_dll(dllname)
     except:
-        raise ChannelAccessException('initialize_libca',
-                                     'Loading Epics CA DLL failed')
+        raise ChannelAccessException('loading Epics CA DLL failed')
 
     ca_context = {False:0, True:1}[PREEMPTIVE_CALLBACK]
     ret = libca.ca_context_create(ca_context)
     if ret != dbr.ECA_NORMAL:
-        raise ChannelAccessException('initialize_libca',
-                                     'Cannot create Epics CA Context')
+        raise ChannelAccessException('cannot create Epics CA Context')
 
     # set argtypes and non-default return types
     # for several libca functions here
@@ -272,12 +254,15 @@ def finalize_libca(maxtime=10.0):
     except StandardError:
         pass
 
+def get_cache(pvname):
+    "return cache dictionary for a given pvname in the current context"
+    return _cache[current_context()].get(pvname, None)
+
 def show_cache(print_out=True):
     """Show list of cached PVs"""
     out = []
     out.append('#  PVName        ChannelID/Context Connected?')
     out.append('#--------------------------------------------')
-    global _cache
     for context, context_chids in  list(_cache.items()):
         for vname, val in list(context_chids.items()):
             chid = val['chid']
@@ -340,9 +325,10 @@ def withCHID(fcn):
             if isinstance(chid, int):
                 args[0] = chid = dbr.chid_t(args[0])
             if not isinstance(chid, dbr.chid_t):
-                raise ChannelAccessException(fcn.__name__,
-                 "not a valid chid %s %s args %s kwargs %s!" %
-                                             (chid, type(chid), args, kwds))
+                msg = "%s: not a valid chid %s %s args %s kwargs %s!" % (
+                    (fcn.__name__, chid, type(chid), args, kwds))
+                raise ChannelAccessException(msg)
+
         return fcn(*args, **kwds)
     wrapper.__doc__ = fcn.__doc__
     wrapper.__name__ = fcn.__name__
@@ -362,8 +348,8 @@ def withConnectedCHID(fcn):
             if isinstance(chid, int):
                 args[0] = chid = dbr.chid_t(chid)
             if not isinstance(chid, dbr.chid_t):
-                raise ChannelAccessException(fcn.__name__,
-                                             "not a valid chid!")
+                raise ChannelAccessException("%s: not a valid chid!" % \
+                                             (fcn.__name__))
             if not isConnected(chid):
                 timeout = kwds.get('timeout', DEFAULT_CONNECTION_TIMEOUT)
                 connect_channel(chid, timeout=timeout)
@@ -408,16 +394,15 @@ def withSEVCHK(fcn):
     return wrapper
 
 ##
-## Event Handlers for get() event callbacks
-def _onGetEvent(args):
-    """Internal Event Handler for get events: not intended for use"""
+## Event Handler for monitor event callbacks
+def _onMonitorEvent(args):
+    """Event Handler for monitor events: not intended for use"""
     value = dbr.cast_args(args).contents
-    # chid = dbr.chid_t(args.chid)
     pvname = name(args.chid)
     kwds = {'ftype':args.type, 'count':args.count,
            'chid':args.chid, 'pvname': pvname,
            'status':args.status}
-    # print("_onGetEvent ", pvname, current_context())
+
     # add kwds arguments for CTRL and TIME variants
     if args.type >= dbr.CTRL_STRING:
         tmpv = value[0]
@@ -439,7 +424,7 @@ def _onGetEvent(args):
     if args.type in (dbr.STRING, dbr.TIME_STRING, dbr.CTRL_STRING):
         nelem = dbr.MAX_STRING_SIZE
 
-    value = _unpack(value, count=nelem, ftype=args.type)
+    value = _unpack(args.chid, value, count=nelem, ftype=args.type)
     if hasattr(args.usr, '__call__'):
         args.usr(value=value, **kwds)
 
@@ -465,7 +450,7 @@ def _onConnectionEvent(args):
 
     if not pv_found:
         _cache[ctx][pvname] = {'conn':False, 'chid': args.chid,
-                               'ts':0, 'failures':0,
+                               'ts':0, 'failures':0, 'value': None,
                                'callbacks': []}
 
     # set connection time, run connection callbacks
@@ -478,17 +463,25 @@ def _onConnectionEvent(args):
                 ichid = entry['chid'].value
 
             if int(ichid) == int(args.chid):
-                entry['conn'] = conn = (args.op == dbr.OP_CONN_UP)
-                entry['chid'] = chid = args.chid
-                entry['ts']   = time.time()
-                entry['failures'] = 0
-
+                conn = (args.op == dbr.OP_CONN_UP)
+                chid = args.chid
+                entry.update({'chid': chid, 'conn': conn,
+                              'ts': time.time(), 'failures': 0})
                 for callback in entry.get('callbacks', []):
                     poll()
                     if hasattr(callback, '__call__'):
                         callback(pvname=pvname, chid=chid, conn=conn)
     return
 
+## get event handler:
+def _onGetEvent(args, **kws):
+    """get_callback event: simply store data contents which
+    will need conversion to python data with _unpack()."""
+    global _cache
+    if args.status != dbr.ECA_NORMAL:
+        return
+    get_cache(name(args.chid))['value'] = memcopy(dbr.cast_args(args).contents)
+
 ## put event handler:
 def _onPutEvent(args, **kwds):
     """set put-has-completed for this channel,
@@ -504,10 +497,11 @@ def _onPutEvent(args, **kwds):
             kwds['data'] = data
         fcn(pvname=pvname, **kwds)
 
-# create global reference to these two callbacks
+# create global reference to these callbacks
 _CB_CONNECT = ctypes.CFUNCTYPE(None, dbr.connection_args)(_onConnectionEvent)
 _CB_PUTWAIT = ctypes.CFUNCTYPE(None, dbr.event_handler_args)(_onPutEvent)
-_CB_EVENT   = ctypes.CFUNCTYPE(None, dbr.event_handler_args)(_onGetEvent)
+_CB_GET     = ctypes.CFUNCTYPE(None, dbr.event_handler_args)(_onGetEvent)
+_CB_EVENT   = ctypes.CFUNCTYPE(None, dbr.event_handler_args)(_onMonitorEvent)
 
 ###
 #
@@ -518,14 +512,15 @@ _CB_EVENT   = ctypes.CFUNCTYPE(None, dbr.event_handler_args)(_onGetEvent)
 # contexts
 @withCA
 @withSEVCHK
-def context_create(context=None):
-    "create a context -- argument ignored in favor of PREEMPTIVE_CALLBACK"
-    ctx = {False:0, True:1}[PREEMPTIVE_CALLBACK]
+def context_create(ctx=None):
+    "create a context. if argument is None, use PREEMPTIVE_CALLBACK"
+    if ctx is None:
+        ctx = {False:0, True:1}[PREEMPTIVE_CALLBACK]
     return libca.ca_context_create(ctx)
 
-def create_context():
+def create_context(ctx):
     "create a context (fixed naming bug)"
-    context_create(context=None)
+    context_create(ctx=ctx)
 
 @withCA
 def context_destroy():
@@ -612,7 +607,7 @@ def pend_io(timeout=1.0):
         return ret
 
 ## @withCA
-def pend_event(timeout=1.e-4):
+def pend_event(timeout=1.e-5):
     """polls CA for events """
     ret = libca.ca_pend_event(timeout)
     try:
@@ -621,7 +616,7 @@ def pend_event(timeout=1.e-4):
         return ret
 
 @withCA
-def poll(evt=1.e-4, iot=1.0):
+def poll(evt=1.e-5, iot=1.0):
     """polls CA for events and i/o. """
     pend_event(evt)
     return pend_io(iot)
@@ -633,19 +628,17 @@ def test_io():
 
 ## create channel
 @withCA
-def create_channel(pvname, connect=False, callback=None):
+def create_channel(pvname, connect=False, auto_cb=True, callback=None):
     """ create a Channel for a given pvname
 
-    connect=True will try to wait until connection is complete
-    before returning
-
-    a user-supplied callback function (callback) can be provided
-    as a connection callback. This function will be called when
-    the connection state changes, and will be passed these keyword
-    arguments:
-       pvname   name of PV
-       chid     channel ID
-       conn     connection state (True/False)
+    connect:  try to wait until connection is complete before returning
+    auto_cb:  use the automatic connection callback
+    callback: a user-supplied callback function (callback) can be provided
+       as a connection callback. This function will be called when the
+       connection state changes, and will be passed these keyword args:
+          pvname   name of PV
+          chid     channel ID
+          conn     connection state (True/False)
 
     If the channel is already connected for the PV name, the callback
     will be called immediately.
@@ -662,7 +655,7 @@ def create_channel(pvname, connect=False, callback=None):
         _cache[ctx] = {}
     if pvname not in _cache[ctx]: # new PV for this context
         entry = {'conn':False,  'chid': None,
-                 'ts': 0,  'failures':0,
+                 'ts': 0,  'failures':0, 'value': None,
                  'callbacks': [ callback ]}
         _cache[ctx][pvname] = entry
     else:
@@ -672,21 +665,25 @@ def create_channel(pvname, connect=False, callback=None):
         elif (hasattr(callback, '__call__') and
               not callback in entry['callbacks']):
             entry['callbacks'].append(callback)
-            callback(chid=entry['chid'], conn=entry['conn'])
+            callback(chid=entry['chid'], pvname=pvname, conn=entry['conn'])
 
+    conncb = 0
+    if auto_cb:
+        conncb = _CB_CONNECT
     if entry.get('chid', None) is not None:
         # already have or waiting on a chid
         chid = _cache[ctx][pvname]['chid']
     else:
         chid = dbr.chid_t()
-        ret = libca.ca_create_channel(pvn, _CB_CONNECT, 0, 0,
-                                  ctypes.byref(chid))
+        ret = libca.ca_create_channel(pvn, conncb, 0, 0,
+                                      ctypes.byref(chid))
         PySEVCHK('create_channel', ret)
         entry['chid'] = chid
 
     if connect:
         connect_channel(chid)
-    poll()
+    if conncb != 0:
+        poll()
     return chid
 
 @withCHID
@@ -719,6 +716,7 @@ def connect_channel(chid, timeout=None, verbose=False):
 
         if timeout is None:
             timeout = DEFAULT_CONNECTION_TIMEOUT
+
         while (not conn and ((time.time()-start_time) < timeout)):
             poll()
             conn = (state(chid) == dbr.CS_CONN)
@@ -799,9 +797,23 @@ def native_type(ftype):
         ntype -= dbr.TIME_STRING
     return ntype
 
-def _unpack(data, count=None, chid=None, ftype=None, as_numpy=True):
+def _unpack(chid, data, count=None, ftype=None, as_numpy=True):
     """unpack raw data returned from an array get or
     subscription callback"""
+
+    def array_cast(data, count, ntype, use_numpy):
+        "cast ctypes array to numpy array (if using numpy)"
+        if use_numpy:
+            dtype = dbr.NP_Map.get(ntype, None)
+            if dtype is not None:
+                out = numpy.empty(shape=(count,), dtype=dbr.NP_Map[ntype])
+                ctypes.memmove(out.ctypes.data, data, out.nbytes)
+            else:
+                out = numpy.ctypeslib.as_array(copy(data))
+        else:
+            out = copy(data)
+        return out
+
     def unpack_simple(data, count, ntype, use_numpy):
         "simple, native data type"
         if count == 1 and ntype != dbr.STRING:
@@ -810,42 +822,29 @@ def _unpack(data, count=None, chid=None, ftype=None, as_numpy=True):
             out = []
             for elem in range(min(count, len(data))):
                 this = strjoin('', data[elem]).rstrip()
-                if '\x00' in this:
-                    this = this[:this.index('\x00')]
+                if NULLCHAR_2 in this:
+                    this = this[:this.index(NULLCHAR_2)]
                 out.append(this)
-
             if len(out) == 1:
-                return out[0]
-            else:
-                return out
-        # waveform data:
-        if ntype == dbr.CHAR:
-            if use_numpy:
-                data = numpy.array(data)
-            return copy.copy(data[:])
-        elif use_numpy:
-            return copy.copy(numpy.ctypeslib.as_array(data))
-        return list(data)
+                out = out[0]
+            return out
+        if count > 1:
+            data = array_cast(data, count, ntype, use_numpy)
+        return data
 
     def unpack_ctrltime(data, count, ntype, use_numpy):
         "ctrl and time data types"
         if count == 1 or ntype == dbr.STRING:
-            out = data[0].value
-            if ntype == dbr.STRING and '\x00' in out:
-                out = out[:out.index('\x00')]
-            return out
+            data = data[0].value
+            if ntype == dbr.STRING and NULLCHAR in data:
+                data = data[:data.index(NULLCHAR)]
+            return data
         # fix for CTRL / TIME array data:Thanks to Glen Wright !
-        out = (count*dbr.Map[ntype]).from_address(ctypes.addressof(data) +
+        data = (count*dbr.Map[ntype]).from_address(ctypes.addressof(data) +
                                                   dbr.value_offset[ftype])
-
-        if ntype == dbr.CHAR:
-            out = [i for i in out]
-            if use_numpy:
-                out = numpy.array(out)
-        if use_numpy:
-            return copy.copy(numpy.ctypeslib.as_array(out))
-
-        return list(out)
+        if count > 1:
+            data = array_cast(data, count, ntype, use_numpy)
+        return data
 
     unpack = unpack_simple
     if ftype >= dbr.TIME_STRING:
@@ -860,43 +859,110 @@ def _unpack(data, count=None, chid=None, ftype=None, as_numpy=True):
         ftype = field_type(chid)
     if ftype is None:
         ftype = dbr.INT
-
     ntype = native_type(ftype)
-    use_numpy = (count > 1 and HAS_NUMPY and as_numpy and
-                 ntype != dbr.STRING)
+    use_numpy = (HAS_NUMPY and as_numpy and ntype != dbr.STRING and count > 1)
     return unpack(data, count, ntype, use_numpy)
 
 @withConnectedCHID
-def get(chid, ftype=None, as_string=False, count=None, as_numpy=True):
+def get(chid, ftype=None, count=None, wait=True, timeout=None,
+        as_string=False, as_numpy=True):
     """return the current value for a Channel.  Options are
        ftype       field type to use (native type is default)
+       count       explicitly limit count
+       wait        flag(True/False) to wait to return value (default) or
+                   return None immediately, with value to be fetched later
+                   by ca.get_complete(chid, ...)
+       timeout     time to wait (and sent to pend_io()) before unpacking
+                   value (default =  0.5 + log10(count) )
        as_string   flag(True/False) to get a string representation
                    of the value returned.  This is not nearly as
                    featured as for a PV -- see pv.py for more details.
        as_numpy    flag(True/False) to use numpy array as the
                    return type for array data.
+
+       get will return None under one of the following conditions:
+         * Channel not connected
+         * wait=False passed in
+         * data is not available within timeout.
     """
     if ftype is None:
         ftype = field_type(chid)
     if ftype in (None, -1):
-        return
+        return None
     if count is None:
         count = element_count(chid)
     else:
         count = min(count, element_count(chid))
 
-    data = (count*dbr.Map[ftype])()
+    ncache = _cache[current_context()][name(chid)]
+    # implementation note: cached value of
+    #   None        implies no value, no expected callback
+    #   GET_PENDING implies no value yet, callback expected.
+    ncache['value'] = GET_PENDING
+
+    uarg = ctypes.py_object(ctypes.POINTER(dbr.Map[ftype]))
+    ret = libca.ca_array_get_callback(ftype, count, chid, _CB_GET, uarg)
 
-    ret = libca.ca_array_get(ftype, count, chid, data)
     PySEVCHK('get', ret)
-    poll()
-    if count > 2:
-        tcount = min(count, 1000)
-        poll(evt=tcount*1.e-5, iot=tcount*0.01)
+    if not wait:
+        return None
+    return get_complete(chid, count=count, ftype=ftype, timeout=timeout,
+                        as_string=as_string, as_numpy=as_numpy)
 
-    val = _unpack(data, count=count, ftype=ftype, as_numpy=as_numpy)
+@withConnectedCHID
+def get_complete(chid, ftype=None, count=None, timeout=None,
+                 as_string=False,  as_numpy=True):
+
+    """returns the value for a channel, completing a previous 'inomplete get' on
+    the channel. This assumes that the previous call to get(chid, ....)  either
+    used 'wait=False' or timed out, and that the data has actually arrived by the
+    time this function is called.
+
+    Important Note: this function can be called only once, as on success, the
+    cached value will be set back to None.
+
+    Options are as for get (but without unpack, which is always True here):
+
+       ftype       field type to use (native type is default)
+       count       explicitly limit count
+       as_string   flag(True/False) to get a string representation
+                   of the value returned.  This is not nearly as
+                   featured as for a PV -- see pv.py for more details.
+       as_numpy    flag(True/False) to use numpy array as the
+                   return type for array data.
+       timeout     time to wait for value to be received
+                   (default = 0.5 + log10(count) seconds)
+   """
+    if ftype is None:
+        ftype = field_type(chid)
+    if count is None:
+        count = element_count(chid)
+    else:
+        count = min(count, element_count(chid))
+
+    ncache = _cache[current_context()][name(chid)]
+    if ncache['value'] is None:
+        return None
+
+    t0 = time.time()
+    if timeout is None:
+        timeout = 1.0 + log10(max(1, count))
+    while ncache['value'] is GET_PENDING:
+        pend_event(1.e-5)
+        if time.time()-t0 > timeout:
+            msg = "ca.get('%s') timed out after %.2f seconds."
+            warnings.warn(msg % (name(chid), timeout))
+            return None
+
+    val = _unpack(chid, ncache['value'], count=count,
+                  ftype=ftype, as_numpy=as_numpy)
     if as_string:
         val = _as_string(val, chid, count, ftype)
+    elif isinstance(val, ctypes.Array) and HAS_NUMPY and as_numpy:
+        val = numpy.array(val)
+
+    # value retrieved, clear cached value
+    ncache['value'] = None
     return val
 
 def _as_string(val, chid, count, ftype):
@@ -934,6 +1000,8 @@ def put(chid, value, wait=False, timeout=30, callback=None,
     """
     ftype = field_type(chid)
     count = element_count(chid)
+    if count > 1: #  and not (ftype == dbr.CHAR and isinstance(value, str)):
+        count = min(len(value), count)
     data  = (count*dbr.Map[ftype])()
 
     if ftype == dbr.STRING:
@@ -948,26 +1016,21 @@ def put(chid, value, wait=False, timeout=30, callback=None,
         except TypeError:
             data[0] = type(data[0])(value)
         except:
-            errmsg = "Cannot put value '%s' to PV of type '%s'"
+            errmsg = "cannot put value '%s' to PV of type '%s'"
             tname  = dbr.Name(ftype).lower()
-            raise ChannelAccessException('put', \
-                                         errmsg % (repr(value),tname))
+            raise ChannelAccessException(errmsg % (repr(value), tname))
+
     else:
-        # auto-convert strings to arrays for character waveforms
-        # could consider using
-        # numpy.fromstring(("%s%s" % (s,'\x00'*maxlen))[:maxlen],
-        #                  dtype=numpy.uint8)
         if ftype == dbr.CHAR and isinstance(value, str):
-            pad = '\x00'*(1+count-len(value))
-            value = [ord(i) for i in ("%s%s" % (value, pad))[:count]]
+            value = [ord(i) for i in ("%s%s" % (value, NULLCHAR))]
         try:
             ndata, nuser = len(data), len(value)
             if nuser > ndata:
                 value = value[:ndata]
             data[:len(value)] = list(value)
         except (ValueError, IndexError):
-            errmsg = "Cannot put array data to PV of type '%s'"
-            raise ChannelAccessException('put', errmsg % (repr(value)))
+            errmsg = "cannot put array data to PV of type '%s'"
+            raise ChannelAccessException(errmsg % (repr(value)))
 
     # simple put, without wait or callback
     if not (wait or hasattr(callback, '__call__')):
@@ -979,19 +1042,17 @@ def put(chid, value, wait=False, timeout=30, callback=None,
     pvname = name(chid)
     _put_done[pvname] = (False, callback, callback_data)
     start_time = time.time()
+    # print "Put:  ", ftype, count
     ret = libca.ca_array_put_callback(ftype, count, chid,
                                       data, _CB_PUTWAIT, 0)
     PySEVCHK('put', ret)
     poll(evt=1.e-4, iot=0.05)
     if wait:
-        finished = False
-        while not finished:
+        while not (_put_done[pvname][0] or
+                   (time.time()-start_time) > timeout):
             poll()
-            finished = (_put_done[pvname][0] or
-                        (time.time()-start_time) > timeout)
         if not _put_done[pvname][0]:
             ret = -ret
-
     return ret
 
 @withConnectedCHID
@@ -1022,7 +1083,7 @@ def get_ctrlvars(chid):
                  'lower_warning_limit','lower_alarm_limit',
                  'upper_ctrl_limit', 'lower_ctrl_limit'):
         if hasattr(tmpv, attr):
-            out[attr] = getattr(tmpv, attr)
+            out[attr] = getattr(tmpv, attr, None)
     if (hasattr(tmpv, 'strs') and hasattr(tmpv, 'no_str') and
         tmpv.no_str > 0):
         out['enum_strs'] = tuple([BYTES2STR(tmpv.strs[i].value)
@@ -1070,17 +1131,27 @@ def get_enum_strings(chid):
         return get_ctrlvars(chid).get('enum_strs', None)
     return None
 
+##
+# Default mask for subscriptions (means update on value changes
+# exceeding MDEL, and on alarm level changes.) Other option is
+# dbr.DBE_LOG for archive changes (ie exceeding ADEL)
+DEFAULT_SUBSCRIPTION_MASK = dbr.DBE_VALUE|dbr.DBE_ALARM
+
 @withConnectedCHID
 def create_subscription(chid, use_time=False, use_ctrl=False,
-                        mask=7, callback=None):
+                        mask=None, callback=None):
     """
     setup a callback function to be called when a PVs value or state changes.
 
+    mask is some combination of dbr.DBE_VALUE, dbr.DBE_ALARM, or default as set above.
+
     Important Note:
         KEEP The returned tuple in named variable: if the return argument
         gets garbage collected, a coredump will occur.
 
     """
+    mask = mask or DEFAULT_SUBSCRIPTION_MASK
+
     ftype = promote_type(chid, use_ctrl=use_ctrl, use_time=use_time)
 
     uarg  = ctypes.py_object(callback)
@@ -1148,7 +1219,7 @@ def sg_get(gid, chid, ftype=None, as_numpy=True, as_string=True):
     >>> print epics.ca._unpack(data, chid=chid)
     """
     if not isinstance(chid, dbr.chid_t):
-        raise ChannelAccessException('sg_get', "not a valid chid!")
+        raise ChannelAccessException("not a valid chid!")
 
     if ftype is None:
         ftype = field_type(chid)
@@ -1159,7 +1230,7 @@ def sg_get(gid, chid, ftype=None, as_numpy=True, as_string=True):
     PySEVCHK('sg_get', ret)
     poll()
 
-    val = _unpack(data, count=count, ftype=ftype, as_numpy=as_numpy)
+    val = _unpack(chid, data, count=count, ftype=ftype, as_numpy=as_numpy)
     if as_string:
         val = _as_string(val, chid, count, ftype)
     return val
@@ -1167,7 +1238,7 @@ def sg_get(gid, chid, ftype=None, as_numpy=True, as_string=True):
 def sg_put(gid, chid, value):
     "synchronous-group put: cannot wait or get callback!"
     if not isinstance(chid, dbr.chid_t):
-        raise ChannelAccessException('sg_put', "not a valid chid!")
+        raise ChannelAccessException("not a valid chid!")
 
     ftype = field_type(chid)
     count = element_count(chid)
@@ -1187,15 +1258,15 @@ def sg_put(gid, chid, value):
         except:
             errmsg = "Cannot put value '%s' to PV of type '%s'"
             tname   = dbr.Name(ftype).lower()
-            raise ChannelAccessException('put', \
-                                         errmsg % (repr(value),tname))
+            raise ChannelAccessException(errmsg % (repr(value), tname))
+
     else:
         # auto-convert strings to arrays for character waveforms
         # could consider using
-        # numpy.fromstring(("%s%s" % (s,'\x00'*maxlen))[:maxlen],
+        # numpy.fromstring(("%s%s" % (s,NULLCHAR*maxlen))[:maxlen],
         #                  dtype=numpy.uint8)
         if ftype == dbr.CHAR and isinstance(value, str):
-            pad = '\x00'*(1+count-len(value))
+            pad = NULLCHAR*(1+count-len(value))
             value = [ord(i) for i in ("%s%s" % (value, pad))[:count]]
         try:
             ndata = len(data)
@@ -1205,7 +1276,7 @@ def sg_put(gid, chid, value):
             data[:len(value)] = list(value)
         except:
             errmsg = "Cannot put array data to PV of type '%s'"
-            raise ChannelAccessException('put', errmsg % (repr(value)))
+            raise ChannelAccessException(errmsg % (repr(value)))
 
     ret =  libca.ca_sg_array_put(gid, ftype, count, chid, data)
     PySEVCHK('sg_put', ret)
