@@ -266,19 +266,18 @@ class Motor(device.Device):
 
     def __getattr__(self, attr):
         " internal method "
-        # print 'GET ATTR ', attr
         if attr in self._alias:
             attr = self._alias[attr]
-        elif attr in self._pvs:
+        if attr in self._pvs:
             return self.get(attr)
-        elif not attr.startswith('__'):
+        if not attr.startswith('__'):
             try:
                 self.PV(attr)
                 return self.get(attr)
             except:
                 raise MotorException("EpicsMotor has no attribute %s" % attr)
         else:
-            return self.__dict__[attr]
+            return self._pvs[attr]
                 
     def __setattr__(self, attr, val):
         # print 'SET ATTR ', attr, val
