@@ -41,7 +41,8 @@ class PV(object):
       >>> p.type           # EPICS data type: 'string','double','enum','long',..
 """
 
-    _fmt = "<PV '%(pvname)s', count=%(count)i, type=%(typefull)s, access=%(access)s>"
+    _fmtsca = "<PV '%(pvname)s', count=%(count)i, type=%(typefull)s, access=%(access)s>"
+    _fmtarr = "<PV '%(pvname)s', count=%(count)i/%(nelm)i, type=%(typefull)s, access=%(access)s>"
     _fields = ('pvname',  'value',  'char_value',  'status',  'ftype',  'chid',
                'host', 'count', 'access', 'write_access', 'read_access',
                'severity', 'timestamp', 'precision', 'units', 'enum_strs',
@@ -64,6 +65,7 @@ class PV(object):
         self._args      = {}.fromkeys(self._fields)
         self._args['pvname'] = self.pvname
         self._args['count'] = -1
+        self._args['nelm']  = -1
         self._args['type'] = 'unknown'
         self._args['typefull'] = 'unknown'
         self._args['access'] = 'unknown'
@@ -73,35 +75,36 @@ class PV(object):
         self.callbacks  = {}
         self._monref = None  # holder of data returned from create_subscription
         self._conn_started = False
+        if isinstance(callback, (tuple, list)):
+            for i, thiscb in enumerate(callback):
+                if hasattr(thiscb, '__call__'):
+                    self.callbacks[i] = (thiscb, {})
+        elif hasattr(callback, '__call__'):
+            self.callbacks[0] = (callback, {})
+
         self.chid = None
-        
         if ca.current_context() is None:
-            ca.use_initial_context() 
+            ca.use_initial_context()
         self.context = ca.current_context()
 
-        self._args['chid'] = self.chid = ca.create_channel(self.pvname,
-                                                           callback=self.__on_connect)
-
+        self._args['chid'] = ca.create_channel(self.pvname,
+                                               callback=self.__on_connect)
+        self.chid = self._args['chid']
         self.ftype  = ca.promote_type(self.chid,
                                       use_ctrl= self.form == 'ctrl',
                                       use_time= self.form == 'time')
-
         self._args['type'] = dbr.Name(self.ftype).lower()
-        
-
-        if callback is not None:
-            self.add_callback(callback)
 
     def __on_connect(self, pvname=None, chid=None, conn=True):
         "callback for connection events"
-        # occassionally chid is still None (ie if a second PV is created while
-        # __on_connect is still pending for the first one.)
+        # occassionally chid is still None (ie if a second PV is created
+        # while __on_connect is still pending for the first one.)
         # Just return here, and connection will happen later
         if self.chid is None and chid is None:
             time.sleep(0.001)
             return
         if conn:
-            self.poll()
+            ca.poll()
             self.chid = self._args['chid'] = dbr.chid_t(chid)
             try:
                 count = ca.element_count(self.chid)
@@ -109,6 +112,7 @@ class PV(object):
                 time.sleep(0.025)
                 count = ca.element_count(self.chid)
             self._args['count']  = count
+            self._args['nelm']  = count
             self._args['host']   = ca.host_name(self.chid)
             self._args['access'] = ca.access(self.chid)
             self._args['read_access'] = (1 == ca.read_access(self.chid))
@@ -124,10 +128,18 @@ class PV(object):
             if self.auto_monitor is None:
                 self.auto_monitor = count < ca.AUTOMONITOR_MAXLENGTH
             if self._monref is None and self.auto_monitor:
+                # you can explicitly request a subscription mask
+                # (ie dbr.DBE_ALARM|dbr.DBE_LOG) by passing it as the
+                # auto_monitor arg, otherwise if you specify 'True' you'll
+                # just get the default set in ca.DEFAULT_SUBSCRIPTION_MASK
+                mask = None
+                if isinstance(self.auto_monitor, int):
+                    mask = self.auto_monitor
                 self._monref = ca.create_subscription(self.chid,
                                          use_ctrl=(self.form == 'ctrl'),
                                          use_time=(self.form == 'time'),
-                                         callback=self.__on_changes)
+                                         callback=self.__on_changes,
+                                         mask=mask)
 
         for conn_cb in self.connection_callbacks:
             if hasattr(conn_cb, '__call__'):
@@ -158,7 +170,7 @@ class PV(object):
             start_time = time.time()
             while (not self.connected and
                    time.time()-start_time < timeout):
-                self.poll()
+                ca.poll()
         return self.connected
 
     def connect(self, timeout=None):
@@ -182,35 +194,58 @@ class PV(object):
         "poll for changes"
         ca.poll(evt=evt, iot=iot)
 
-    def get(self, count=None, as_string=False, as_numpy=True):
+    def get(self, count=None, as_string=False, as_numpy=True,
+            timeout=None, with_ctrlvars=False, use_monitor=True):
         """returns current value of PV.  Use the options:
-         as_string to return string representation
-         as_numpy  to (try to) return a numpy array
+        count       explicitly limit count for array data
+        as_string   flag(True/False) to get a string representation
+                    of the value.
+        as_numpy    flag(True/False) to use numpy array as the
+                    return type for array data.
+        timeout     maximum time to wait for value to be received.
+                    (default = 0.5 + log10(count) seconds)
+        use_monitor flag(True/False) to use value from latest
+                    monitor callback (True, default) or to make an
+                    explicit CA call for the value.
 
         >>> p.get('13BMD:m1.DIR')
         0
-        >>> p.get('13BMD:m1.DIR',as_string=True)
+        >>> p.get('13BMD:m1.DIR', as_string=True)
         'Pos'
         """
         if not self.wait_for_connection():
             return None
+        if with_ctrlvars and getattr(self, 'units', None) is None:
+            self.get_ctrlvars()
 
-        if not self.auto_monitor or self._args['value'] is None:
-            self._args['value'] = ca.get(self.chid,
-                                         count=count,
-                                         ftype=self.ftype,
+        if ((not use_monitor) or
+            (not self.auto_monitor) or
+            (self._args['value'] is None) or
+            (count is not None and count > len(self._args['value']))):
+            ca_get = ca.get
+            if ca.get_cache(self.pvname)['value'] is not None:
+                ca_get = ca.get_complete
+            self._args['value'] = ca_get(self.chid, ftype=self.ftype,
+                                         count=count, timeout=timeout,
                                          as_numpy=as_numpy)
-
+        val = self._args['value']
         if as_string:
-            self._set_charval(self._args['value'])
-            return self._args['char_value']
+            return self._set_charval(val)
+        if self.count <= 1 or val is None:
+            return val
 
-        # this emulates asking for less data than actually exists in the
-        # cached value
-        if count is not None and len(self._args['value']) > 1:
-            count = max(0, min(count, len(self._args['value'])))
-            return self._args['value'][:count]
-        return self._args['value']
+        if count is None:
+            count = len(val)
+        if (as_numpy and ca.HAS_NUMPY and count > 1 and
+            not isinstance(val, ca.numpy.ndarray)):
+            val = ca.numpy.array(val)
+        elif (not as_numpy and ca.HAS_NUMPY and
+              isinstance(val, ca.numpy.ndarray)):
+            val = list(val)
+        # allow asking for less data than actually exists in the cached value
+        if count < len(val):
+            val = val[:count]
+        return val
 
     def put(self, value, wait=False, timeout=30.0,
             use_complete=False, callback=None, callback_data=None):
@@ -220,12 +255,18 @@ class PV(object):
         """
         if not self.wait_for_connection():
             return None
+
         if (self.ftype in (dbr.ENUM, dbr.TIME_ENUM, dbr.CTRL_ENUM) and
             isinstance(value, str)):
             if self._args['enum_strs'] is None:
                 self.get_ctrlvars()
             if value in self._args['enum_strs']:
-                value = self._args['enum_strs'].index(value)
+                # tuple.index() not supported in python2.5
+                # value = self._args['enum_strs'].index(value)
+                for ival, val in enumerate(self._args['enum_strs']):
+                    if val == value:
+                        value = ival
+                        break
         if use_complete and callback is None:
             callback = self.__putCallbackStub
         return ca.put(self.chid, value,
@@ -240,6 +281,9 @@ class PV(object):
     def _set_charval(self, val, call_ca=True):
         """ sets the character representation of the value.
         intended only for internal use"""
+        if val is None:
+            self._args['char_value'] = 'None'
+            return 'None'
         ftype = self._args['ftype']
         ntype = ca.native_type(ftype)
         if ntype == dbr.STRING:
@@ -285,13 +329,22 @@ class PV(object):
 
     def get_ctrlvars(self):
         "get control values for variable"
+        return self._get_vars(ca.get_ctrlvars)
+
+    def get_timevars(self):
+        "get time values for variable"
+        return self._get_vars(ca.get_timevars)
+
+    def _get_vars(self, var_fn):
+        "internal, common functionality for retreiving control/times values"
         if not self.wait_for_connection():
             return None
-        kwds = ca.get_ctrlvars(self.chid)
+        kwds = var_fn(self.chid)
         ca.poll()
         self._args.update(kwds)
         return kwds
 
+
     def __on_changes(self, value=None, **kwd):
         """internal callback function: do not overwrite!!
         To have user-defined code run when the PV value changes,
@@ -301,12 +354,10 @@ class PV(object):
         self._args['value']  = value
         self._args['timestamp'] = kwd.get('timestamp', time.time())
         self._set_charval(self._args['value'], call_ca=False)
-
         if self.verbose:
             now = fmt_time(self._args['timestamp'])
             ca.write('%s: %s (%s)'% (self.pvname,
-                                     self._args['char_value'],
-                                     now))
+                                     self._args['char_value'], now))
         self.run_callbacks()
 
     def run_callbacks(self):
@@ -315,7 +366,13 @@ class PV(object):
         Normally, this is to be run automatically on event, but
         it is provided here as a separate function for testing
         purposes.
+        """
+        for index in sorted(list(self.callbacks.keys())):
+            self.run_callback(index)
 
+    def run_callback(self, index):
+        """run a specific user-defined callback, specified by index,
+        with the current data
         Note that callback functions are called with keyword/val
         arguments including:
              self._args  (all PV data available, keys = __fields)
@@ -324,17 +381,15 @@ class PV(object):
         where the 'cb_info' is provided as a hook so that a callback
         function  that fails may de-register itself (for example, if
         a GUI resource is no longer available).
-
         """
-        for index in sorted(list(self.callbacks.keys())):
-            fcn, kwargs = self.callbacks[index]
-            kwd = copy.copy(self._args)
-            kwd.update(kwargs)
-            kwd['cb_info'] = (index, self)
-            if hasattr(fcn, '__call__'):
-                fcn(**kwd)
+        fcn, kwargs = self.callbacks[index]
+        kwd = copy.copy(self._args)
+        kwd.update(kwargs)
+        kwd['cb_info'] = (index, self)
+        if hasattr(fcn, '__call__'):
+            fcn(**kwd)
 
-    def add_callback(self, callback=None, index=None,
+    def add_callback(self, callback=None, index=None, run_now=False,
                      with_ctrlvars=True, **kw):
         """add a callback to a PV.  Optional keyword arguments
         set here will be preserved and passed on to the callback
@@ -343,23 +398,27 @@ class PV(object):
         Note that a PV may have multiple callbacks, so that each
         has a unique index (small integer) that is returned by
         add_callback.  This index is needed to remove a callback."""
-        if not self.wait_for_connection():
-            return None
-        if with_ctrlvars:
-            self.get_ctrlvars()
         if hasattr(callback, '__call__'):
             if index is None:
                 index = 1
                 if len(self.callbacks) > 0:
                     index = 1 + max(self.callbacks.keys())
             self.callbacks[index] = (callback, kw)
+
+        if with_ctrlvars and self.connected:
+            self.get_ctrlvars()
+        if run_now:
+            if not self.connected:
+                self.wait_for_connection()
+        if run_now and self.connected:
+            self.run_callback(index)
         return index
 
     def remove_callback(self, index=None):
         """remove a callback by index"""
         if index in self.callbacks:
             self.callbacks.pop(index)
-            self.poll()
+            ca.poll()
 
     def clear_callbacks(self):
         "clear all callbacks"
@@ -390,10 +449,14 @@ class PV(object):
         else:
             ext  = {True:'...', False:''}[self.count > 10]
             elems = range(min(5, self.count))
-            aval = [fmt % self._args['value'][i] for i in elems]
+            try:
+                aval = [fmt % self._args['value'][i] for i in elems]
+            except TypeError:
+                aval = ('unknown',)
             out.append("   value      = array  [%s%s]" % (",".join(aval), ext))
-        for nam in ('char_value', 'count', 'type', 'units', 'precision',
-                    'host', 'access', 'status', 'severity', 'timestamp',
+        for nam in ('char_value', 'count', 'nelm', 'type', 'units',
+                    'precision', 'host', 'access',
+                    'status', 'severity', 'timestamp',
                     'upper_ctrl_limit', 'lower_ctrl_limit',
                     'upper_disp_limit', 'lower_disp_limit',
                     'upper_alarm_limit', 'lower_alarm_limit',
@@ -471,8 +534,21 @@ class PV(object):
 
     @property
     def count(self):
-        "count (number of elements)"
-        return self._getarg('count')
+        """count (number of elements). For array data and later EPICS versions,
+        this is equivalent to the .NORD field.  See also 'nelm' property"""
+        if self._args['count'] >=0:
+            return self._args['count']
+        else:
+            return self._getarg('count')
+
+    @property
+    def nelm(self):
+        """native count (number of elements).
+        For array data this will return the full array size (ie, the
+        .NELM field).  See also 'count' property"""
+        if self._getarg('count') == 1:
+            return 1
+        return ca.element_count(self.chid)
 
     @property
     def read_access(self):
@@ -571,7 +647,10 @@ class PV(object):
         "string representation"
 
         if self.connected:
-            return self._fmt % self._args
+            if self.count == 1:
+                return self._fmtsca % self._args
+            else:
+                return self._fmtarr % self._args
         else:
             return "<PV '%s': not connected>" % self.pvname
 
