@@ -4,8 +4,7 @@ Epics Controls.  In general,  these controls combine a wx widget with
 an Epics PV, and allow automatic updating of the widget when the
 associated PV changes.
 """
-from . import motorpanel
-from . import wxlib, ogllib, utils
+from . import motorpanel, wxlib, ogllib, utils
 
 MotorPanel    = motorpanel.MotorPanel
 
@@ -21,6 +20,7 @@ PVFloatSpin   = pvFloatSpin   = wxlib.PVFloatSpin
 PVButton      = pvButton      = wxlib.PVButton
 PVRadioButton = pvRadioButton = wxlib.PVRadioButton
 PVComboBox    = pvComboBox    = wxlib.PVComboBox
+PVCollapsiblePane = pvCollapsiblePane = wxlib.PVCollapsiblePane
 
 # OGL shapes
 PVRectangle   = pvRectangle   = ogllib.PVRectangle
@@ -36,3 +36,4 @@ DelayedEpicsCallback = wxlib.DelayedEpicsCallback
 EpicsFunction  = wxlib.EpicsFunction
 finalize_epics  = wxlib.finalize_epics
 EpicsTimer      = wxlib.EpicsTimer
+
