@@ -17,7 +17,7 @@ prefix = 'Py:'
 global NEEDS_INIT
 
 NEEDS_INIT = True
-SLEEP_TIME = 0.25
+SLEEP_TIME = 0.10
 
 def onConnect(pvname=None, conn=None, **kws):
     global NEEDS_INIT
@@ -25,8 +25,8 @@ def onConnect(pvname=None, conn=None, **kws):
 
         
 def make_pvs(*args, **kwds):
-    #print "Make PVS '  ", prefix,  args
-    #print  [("%s%s" % (prefix, name)) for name in args]
+    # print "Make PVS '  ", prefix,  args
+    # print  [("%s%s" % (prefix, name)) for name in args]
     pvlist = [epics.PV("%s%s" % (prefix, name)) for name in args]
     for pv in pvlist:
         pv.connect()
@@ -120,21 +120,22 @@ while True:
     if pause_pv.get() == 1:
         # pause for up to 15 seconds if pause was selected
         t0 = time.time()
-        time.sleep(10)
         while time.time()-t0 < 15:
-            time.sleep(0.25)
+            time.sleep(SLEEP_TIME)
             if pause_pv.get() == 0:
                 break
-            
+    noise = numpy.random.normal
     analogs[0].put( 100*(random.random()-0.5))
     analogs[1].put( 76.54321*(time.time()-start_time))
-    analogs[2].put( numpy.sqrt(count*7123.) % 100)
+    analogs[2].put( 0.3*numpy.sin(time.time() / 2.302) + noise(scale=0.4)  )
+    analogs[3].put( numpy.exp( (max(0.001,  noise(scale=0.03) + numpy.sqrt((count/16.0) % 87.)))))
+
     if t0-long_update >= 1.0:
         long_update=t0
         lcount = (lcount + 1) % 10
         longs[0].put(lcount)
         print ' >> ', text[lcount]
         char_waves[1].put(text[lcount])
-
-
+        double_waves[2].put([random.random() for i in range(65536)])
+        double_waves[1].put([random.random() for i in range(2048)])
 
